/**
 * @name		jQuery Countdown Plugin
 * @author		Martin Angelov
 * @version 	1.0
 * @url			http://tutorialzine.com/2011/12/countdown-jquery/
 * @license		MIT License
 */

(function($){
	
	// Number of seconds in every time division
	var days	= 24*60*60,
		hours	= 60*60,
		minutes	= 60,
		timer = null;
		
	// Creating the plugin
	$.fn.stopCountdown = function(){
		clearTimeout(timer);
		
		return this;
	};
	
	// Creating the plugin
	$.fn.countdown = function(prop){
		
		var make = function() {
			
			var $this = $(this);
			
			var options = $.extend({
				onTick	: function(){},
				onFinish	: function(){},
				timestamp	: 0,
				regional: {
					labels: ['Days', 'Hours', 'Minutes', 'Seconds'],
					labels1: ['Day', 'Hour', 'Minute', 'Second'],
					labels2: ['Days', 'Hours', 'Minutes', 'Seconds']
				},
				animation: 'slide_down' // slide_down
			},prop);
			
			var left, d, h, m, s, positions;
	
			// Initialize the plugin
			init($this, options);
			
			positions = $this.find('.position');
			labels = $this.find('.label');
			
			(function tick(){
				
				// Time left
				left = options.timestamp--;
				
				if(left < 0){
					left = 0;
					options.onFinish();
				}
	
				// Number of days left
					d = Math.floor(left / days);
					updateDuo(0, 1, d);
					left -= d*days;
				
					// update labels
					updateLabels(0,d);
				
				// Number of hours left
					h = Math.floor(left / hours);
					updateDuo(2, 3, h);
					left -= h*hours;
					
					// update labels
					updateLabels(1,h);
				
				// Number of minutes left
					m = Math.floor(left / minutes);
					updateDuo(4, 5, m);
					left -= m*minutes;
					
					// update labels
					updateLabels(2,m);
				
				// Number of seconds left
					s = left;
					updateDuo(6, 7, s);
				
					// update labels
					updateLabels(3,s);
	
				// Calling an optional user supplied callback
				options.onTick(d, h, m, s);
				
				// Scheduling another call of this function in 1s
				timer = setTimeout(tick, 1000);
			})();
			
			// hiding first separator
				$(".countdown > div:visible:first").each(function(){
					if ($(this).hasClass("countDiv"))
						$(this).hide();
				});
			
			// This function updates two digit positions at once
			function updateDuo(minor,major,value){
				switchDigit(positions.eq(minor),Math.floor(value/10)%10, options);
				switchDigit(positions.eq(major),value%10, options);
			}
			
			function updateLabels(unit, s)
			{
				var label = options.regional['labels'+whichLabel(s)][unit];
				labels.eq(unit).text(label);
			}
		};
		
		return this.each(make);
	};




	function init(elem, options){
		elem.addClass('countdown');


		elem.css("width", (elem.parent().width() - 100)+'px');
		elem.css("padding","0");

		// Creating the markup inside the container
		parts = ['Days','Hours','Minutes','Seconds'];
		$.each(parts,function(i){
			$('<div class="count'+this+' '+this+' near">').html(
				'<div class="digits">\
					<div class="position">\
						<div class="digit static">0</div>\
					</div>\
					<div class="position">\
						<div class="digit static">0</div>\
					</div>\
					<div class="clear"></div>\
				</div>\
				<div class="label '+this+'"></div>\
				'
			).appendTo(elem);
			
			if(this!="Seconds"){
				elem.append('<div class="countDiv '+parts[i+1]+' countDiv'+i+'"></div>');
			}
		});

	}
	
	function whichLabel(amount) {
		var units = amount % 10;
		var tens = Math.floor((amount % 100) / 10);
		return (amount == 1 ? 1 : (units >= 2 && units <= 4 && tens != 1 ? 2 :
			(units == 1 && tens != 1 ? 1 : '')));
	}
	
	
	
	

	// Creates an animated transition between the two numbers
	function switchDigit(position, number, options){
		
		var digit = position.find('.digit')
		
		if(digit.is(':animated')){
			return false;
		}
		
		if(position.data('digit') == number){
			// We are already showing this number
			return false;
		}
		
		position.data('digit', number);
		
		switch(options.animation)
		{
		
			case 'slide_up':
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						top:'2.1em',
						opacity:0
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({top:'-2.5em',opacity:0},400,function(){
						digit.remove();
					})
		
				replacement
					.delay(100)
					.animate({top:0,opacity:1},400,function(){
						replacement.addClass('static');
					});
			break;
		
			case 'jumping':
				
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						top:'2.1em',
						opacity:0
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({top:'2.5em',opacity:0},400,function(){
						digit.remove();
					})
		
				replacement
					.delay(100)
					.animate({top:0,opacity:1},400,function(){
						replacement.addClass('static');
					});
			break;
		
		
			case 'blind':
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						top:'-2.1em',
						opacity:0
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({top:'-2.5em',opacity:0},400,function(){
						digit.remove();
					})
		
				replacement
					.delay(100)
					.animate({top:0,opacity:1},400,function(){
						replacement.addClass('static');
					});
			break;
		
		
			case 'shuffling':
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						top:'-2.1em',
						right: '-2em',
						opacity:1,
						'z-index': 11
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({},400,function(){
						
					});
					
				digit.css("z-index","10");
		
				replacement
					.delay(100)
					.animate({top:0,right:1,opacity:1},400,function(){
						replacement.addClass('static').css("z-index","10");
						digit.remove();
					});
			break;
		
		
			case 'blink':
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						opacity:0
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({opacity:0},400,function(){
						digit.remove();
					})
		
				replacement
					.delay(100)
					.animate({opacity:1},400,function(){
						replacement.addClass('static');
					});
			break;
		
		
			case 'santa_barbara':
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						height:'0.1em',
						width:'0.1em',
						opacity:0,
						bottom: '2px',
						left: '50%'
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({opacity:0},400,function(){
						digit.remove();
					})
		
				replacement
					.delay(100)
					.animate({left:'0', height:'1.5em',width:'1em',opacity:1},400,function(){
						replacement.addClass('static');
					});
			break;


			case 'waterdrops':
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						top:'0',
						opacity:0
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({top:'5em',opacity:0},500,function(){
						digit.remove();
					})
		
				replacement
					.delay(200)
					.animate({top:0,opacity:1},500,function(){
						replacement.addClass('static');
					});
			break;
		
			//case 'slide_down':
			default:
				var replacement = $('<span>',{
					'class':'digit',
					css:{
						top:'-2.1em',
						opacity:0
					},
					html:number
				});
				
				// The .static class is added when the animation
				// completes. This makes it run smoother.
				
				digit
					.before(replacement)
					.removeClass('static')
					.animate({top:'2.5em',opacity:0},400,function(){
						digit.remove();
					})
		
				replacement
					.delay(100)
					.animate({top:0,opacity:1},400,function(){
						replacement.addClass('static');
					});
			break;
		
		}
		
	}
})(jQuery);