<?php

	class Megaticker_model {
		
		public $wpdb;
		private $table;
		private $order_by = '';
		private $limit = '';
		private $join = array();
		private $select = '*';
		public $insert_id = null;
		
		public function order_by($field, $order)
		{
			$this->order_by = 'ORDER BY '.$field.' '.strtoupper($order);
		}
		
		public function limit($value, $offset = '')
		{
			$this->limit = 'LIMIT '.$value;
			
			if ($offset)
				$this->limit .= ' OFFSET '.$offset;
		}
		
		public function join($table, $condition, $type = '')
		{
			$this->join[] = strtoupper($type).' JOIN '.$table.' ON '.$condition;
		}
		
		public function select($select)
		{
			$this->select = $select;
		}
		
		
		
		function __construct($table)
		{
			global $wpdb;
			$this->wpdb = $wpdb;
			$this->table = $table;
		}
		
		function getAll()
		{
			$sql = "
				SELECT ".$this->select."
				FROM `".$this->wpdb->prefix.$this->table."`";
				
			if (count($this->join))
				$sql .= " ".implode(' ',$this->join)." ";
			
			$sql .= " ".$this->order_by."
				".$this->limit;
				
			$result = $this->wpdb->get_results($sql, ARRAY_A);
			//die($this->wpdb->last_query);
			return $result;
		}
		
		function countAll()
		{
			$sql = "
				SELECT COUNT(id) as count
				FROM `".$this->wpdb->prefix.$this->table."`
				";
				
			$row = $this->wpdb->get_row($sql, ARRAY_A);
			return $row['count'];
		}
		
		
		
		
		function getOne($id = '')
		{
			$data = null;
			if ($id)
			{
				$sql = "SELECT * FROM `".$this->wpdb->prefix.$this->table."` WHERE `id` = ".$id;
				$data = $this->wpdb->get_row($sql, ARRAY_A);
			}
			
			if (is_null($data))
			{
				$sql = "SHOW COLUMNS FROM ".$this->wpdb->prefix.$this->table;
				$columns = $this->wpdb->get_results($sql, ARRAY_A);
				$data = array();
				foreach($columns as $col)
					$data[$col['Field']] = '';
			}
			
			return $data;
		}
		
		
		public static function getTicker($id = '')
		{
			global $wpdb;
			
			$data = null;
			if ($id)
			{
				$sql = "
					SELECT
						`".$wpdb->prefix."megaticker_campaigns`.*,
						
						`".$wpdb->prefix."megaticker_splashes`.`name` as splash_name,
						`".$wpdb->prefix."megaticker_splashes`.`type` as splash_type,
						`".$wpdb->prefix."megaticker_splashes`.`url` as splash_url,
						`".$wpdb->prefix."megaticker_splashes`.`css` as splash_css,
						
						`".$wpdb->prefix."megaticker_counters`.`name` as counter_name,
						`".$wpdb->prefix."megaticker_counters`.`params` as counter_params
					FROM `".$wpdb->prefix."megaticker_campaigns`
					LEFT JOIN `".$wpdb->prefix."megaticker_splashes` ON `splash_id` = `".$wpdb->prefix."megaticker_splashes`.`id`
					LEFT JOIN `".$wpdb->prefix."megaticker_counters` ON `counter_id` = `".$wpdb->prefix."megaticker_counters`.`id`
					WHERE `".$wpdb->prefix."megaticker_campaigns`.`id` = ".$id;
				$data = $wpdb->get_row($sql, ARRAY_A);

			}
			
			if (is_null($data))
			{
				$sql = "SHOW COLUMNS FROM `".$wpdb->prefix."megaticker_counters`";
				$columns = $wpdb->get_results($sql, ARRAY_A);
				$data = array();
				foreach($columns as $col)
					$data[$col['Field']] = '';
					
				$data = array_merge($data,array(
					'splash_name',
					'splash_type',
					'splash_url',
					'counter_name',
					'counter_params'
				));
			}
			
			return $data;
		}
		
		
		public static function getCampaigns()
		{
			global $wpdb;
			
			$sql = "
				SELECT `".$wpdb->prefix."megaticker_campaigns`.*,
						
						`".$wpdb->prefix."megaticker_splashes`.`name` as splash_name,
						`".$wpdb->prefix."megaticker_splashes`.`type` as splash_type,
						`".$wpdb->prefix."megaticker_splashes`.`url` as splash_url,
						`".$wpdb->prefix."megaticker_splashes`.`css` as splash_css
						
				FROM `".$wpdb->prefix."megaticker_campaigns`
				LEFT JOIN `".$wpdb->prefix."megaticker_splashes` ON `splash_id` = `".$wpdb->prefix."megaticker_splashes`.`id`
			";
			return $wpdb->get_results($sql, ARRAY_A);
		}
		
		function getList()
		{
			$sql = "SELECT id,name FROM `".$this->wpdb->prefix.$this->table."` ";
			return $this->wpdb->get_results($sql, ARRAY_A);
		}
		
		function copy($id = '', $new_name = '')
		{
			
			if (!$id)
				return false;
			
			$sql = "SHOW COLUMNS FROM ".$this->wpdb->prefix.$this->table;
			$columns = $this->wpdb->get_results($sql, ARRAY_A);
			$defaults = array();
			foreach($columns as $col)
				$defaults[$col['Field']] = '';
		
			$sql = "SELECT * FROM `".$this->wpdb->prefix.$this->table."` WHERE `id` = ".$id;
			$data = $this->wpdb->get_row($sql, ARRAY_A);
			$data = array_merge($defaults,$data);
			
			unset($data['id']);
			
			if (isset($data['name']))
			{
				if ($new_name)
					$data['name'] = $new_name;
				else
					$data['name'] = $data['name'].'_'.__('copy', 'megaticker');
			}
			
			
			$formats = array();
			foreach($data as $value)
			{
				if (is_numeric($value))
					$formats[] = '%d';
				else
					$formats[] = '%s';
			}
			
			// copying
			return $this->wpdb->insert($this->wpdb->prefix.$this->table, $data, $formats);
		}
		
		
		function delete($ids = '')
		{
			if (!$ids or !is_array($ids) or !count($ids))
				return false;
			
			$sql = "DELETE FROM `".$this->wpdb->prefix.$this->table."` WHERE `id` IN (".implode(',',$ids).")";
			return $this->wpdb->query($sql);
		}
		
		function insert($data, $formats) {
			$result = $this->wpdb->insert($this->wpdb->prefix.$this->table, $data, $formats);
			$this->insert_id = $this->wpdb->insert_id;
			return $result;
		}
		
		function update($data, $where) {
			$this->wpdb->update($this->wpdb->prefix.$this->table, $data, $where);
			
			if ($this->wpdb->print_error())
				$result = false;
			else
				$result = true;
					
			return $result;
		}
		
		
		/* SESSIONS */
		
			public static function getSession($cookie)
			{
				global $wpdb;
				
				$sql = "
					SELECT `".$wpdb->prefix."megaticker_sessions_data`.*,
						`".$wpdb->prefix."megaticker_sessions`.`id` as user_id,
						`".$wpdb->prefix."megaticker_sessions`.`cookie` as user_cookie,
						`".$wpdb->prefix."megaticker_campaigns`.`campaign_type` as campaign_type
					FROM
						`".$wpdb->prefix."megaticker_sessions`
					LEFT JOIN
						`".$wpdb->prefix."megaticker_sessions_data` ON `session_id` = `".$wpdb->prefix."megaticker_sessions`.`id`
					LEFT JOIN
						`".$wpdb->prefix."megaticker_campaigns` ON `".$wpdb->prefix."megaticker_campaigns`.`id` = `".$wpdb->prefix."megaticker_sessions_data`.`campaign_id`
					WHERE
						`cookie` = '".$cookie."'
				";

				$results = $wpdb->get_results($sql, ARRAY_A);

				$finalResults = array();
				foreach($results as $r) {
					
					if ($r['campaign_type'] == 'individual' or $r['campaign_type'] == 'first_visit_only')
						$finalResults[$r['campaign_id'].'_'.$r['url']] = $r;
					else
						$finalResults[$r['campaign_id']] = $r;
				}
					
				return array(
					'user' => array(
						'id' => $results[0]['user_id'],
						'cookie' => $results[0]['user_cookie'],
					),
					'visits' => $finalResults
				);
			}
			
			
			public static function startSession()
			{
				$newCookie = md5(session_id().time());
				
				global $wpdb;
				
				$result = $wpdb->insert($wpdb->prefix."megaticker_sessions", array('cookie' => $newCookie), array('%s'));
				
				if ($result)
					return array(
						'id' => $wpdb->insert_id,
						'cookie' => $newCookie
					);
				else
					return false;
			}
			
			
			public static function addSessionData($campaign_id)
			{
			
				global $wpdb;
				
				$formats[] = '%d';
				$data['session_id'] = $_SESSION['megaticker']['user']['id'];
				
				$formats[] = '%d';
				$data['campaign_id'] = $campaign_id;
				
				$formats[] = '%s';
				$data['url'] = md5(MegaTicker_FormSaver::current_url());
				
				$formats[] = '%d';
				$data['created'] = time();
				
				$result = $wpdb->insert($wpdb->prefix."megaticker_sessions_data", $data, $formats);
				
				$data['id'] = $wpdb->insert_id;
				
				if ($result)
					return $data;
				else
					return false;
			}
			
		
	}